/*!
* \file ElementaryPath.h
* \brief Elementary path determination (3D -> 2D, convex hull)
* \author CSTB
* \version 1.0
*/

#ifndef ELEMENTARY_PATH_H
#define ELEMENTARY_PATH_H

#include <math.h>
#include <stdio.h>
#include "../PathStructures.h"

namespace ElementaryPathNMPB
{
	/*!
	* \brief Finds the convex hull of the screenItems vector
	* 
	* p.47-49 -  9.4.3
	*
	* \param pathItems The points to find the convex hull
	* 
	* \param n1 The first element place in the vector
	* 
	* \param n2 The last element place in the vector
	* 
	* \param level The convex hull level
	*/
	void ConvexHull(vector<ProfilePointNMPB*>& pathItem, int n1, int n2, int level);

	/*!
	* \brief Sets the elementary path for the given path (calculates convex hull and plane positions)
	* 
	* \param path The Propagation path
	* 
	* \throws ERRNoPoint
	* 
	* \throws ERROnePoint
	*/
	void SetElementaryPath(PropagationPath* path);
}

#endif